<?php

namespace Drupal\ckeditor5_embedded_msforms\Plugin\EmbeddedContent;

use Drupal\ckeditor5_embedded_content\EmbeddedContentInterface;
use Drupal\ckeditor5_embedded_content\EmbeddedContentPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * MS Forms plugin.
 *
 * @see https://forms.office.com
 * @see https://support.microsoft.com/en-us/forms
 *
 * @EmbeddedContent(
 *   id = "msforms",
 *   label = @Translation("Microsoft Forms"),
 *   description = @Translation("Renders a Microsoft Form."),
 * )
 */
class MsForms extends EmbeddedContentPluginBase implements EmbeddedContentInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'url' => NULL,
      'width' => NULL,
      'height' => NULL,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    return [
      '#theme' => 'ckeditor5_embedded_msforms',
      '#url' => $this->configuration['url'],
      '#width' => $this->configuration['width'],
      '#height' => $this->configuration['height'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['url'] = [
      '#type' => 'url',
      '#title' => $this->t('URL'),
      '#default_value' => $this->configuration['url'],
      '#required' => TRUE,
      '#description' => $this->t('The URL of the form, e.g. https://forms.office.com/Pages/ResponsePage.aspx?id=abcd or https://forms.office.com/r/xyz'),
    ];
    $form['width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Width'),
      '#default_value' => $this->configuration['width'],
      '#required' => TRUE,
      '#description' => $this->t('Iframe width, in pixels or as a percentage of the available area. Examples: 640px, 100%.'),
      '#size' => 10,
      '#maxlength' => 10,
    ];
    $form['height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Height'),
      '#default_value' => $this->configuration['height'],
      '#required' => TRUE,
      '#description' => $this->t('Iframe height, in pixels or as a percentage of the available area. Examples: 480px, 100%.'),
      '#size' => 10,
      '#maxlength' => 10,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Validate form URL.
    $pattern = '/^https:\/\/forms\.office\.com\/((r\/[\w-]+)|(Pages\/ResponsePage\.aspx\?id=[\w-]+))$/';
    $url = $form_state->getValue('url');
    if (!preg_match($pattern, $url)) {
      $form_state->setErrorByName('url', $this->t('The given URL is not a Microsoft Form.'));
    }

    // Validate width.
    $width = $form_state->getValue('width');
    if (!$this->isValidLength($width)) {
      $form_state->setErrorByName('width', $this->t('Invalid width.'));
    }

    // Validate height.
    $height = $form_state->getValue('height');
    if (!$this->isValidLength($height)) {
      $form_state->setErrorByName('height', $this->t('Invalid height.'));
    }
  }

  /**
   * Validate whether a value is a valid CSS length, in pixels or as percentage.
   *
   * This validates: 640px, 100%, 80.
   * This does not: 640pixel, 3cm.
   *
   * @param string $lengthValue
   *   Value.
   *
   * @return bool
   *   Whether the value is a valid CSS length.
   */
  private function isValidLength($lengthValue) {
    if (str_ends_with($lengthValue, '%')) {
      $lengthValue = substr($lengthValue, 0, -1);
    }
    elseif (str_ends_with($lengthValue, 'px')) {
      $lengthValue = substr($lengthValue, 0, -2);
    }
    return is_numeric($lengthValue);
  }

}
